#!/usr/bin/python
# -*- coding: utf-8 -*-

#skoreboard
#skoreboard fetches soccer results and leagues of a specified country or cup from the website kicker.de and displays them in a nice manner.

    #Copyright (C) 2008  Arthur Schiwon <kontakt@arthur-schiwon.de>

    #This program is free software: you can redistribute it and/or modify
    #it under the terms of the GNU General Public License as published by
    #the Free Software Foundation, either version 3 of the License, or
    #(at your option) any later version.

    #This program is distributed in the hope that it will be useful,
    #but WITHOUT ANY WARRANTY; without even the implied warranty of
    #MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    #GNU General Public License for more details.

    #You should have received a copy of the GNU General Public License
    #along with this program.  If not, see <http://www.gnu.org/licenses/>.

import scoreboard
import sys
import os.path
from PyQt4 import QtGui, QtCore
from PyKDE4 import kdeui, kdecore
from main import Ui_MainWindow as MainDlg

version = "0.1.2"
appname = "Skoreboard"
catalog = ""

class SkbHandler():
	scbd = None
	datarow = None
	
	def __init__(self, scbd):
		self.scbd = scbd
		
	def updateScore(self, tblData, storage, storInd):
		table = []
		prfName = self.scbd.getProfile()
		if(prfName in storInd):
			index = storInd[prfName]
			i = 0
			for tr in tblData:
				row = self.datarow.copy()
				row["changed"] = []
				change = []
				for key in tr.keys():
					if(storage[index][i][key] != tr[key]):
						row["changed"].append(key)
					row[key] = tr[key]
				table.append(row)
				i += 1
			storage[index] = table
		else:
			for tr in tblData:
				row = self.datarow.copy()
				for key in self.datarow.keys():
					row[key] = tr[key]
				row["changed"] = []
				table.append(row)
			storage.append(table)
			storInd[prfName] = len(storage) - 1
		

class SbdMatch(SkbHandler):
	matches = []
	profiles = {}
	
	def __init__(self, scbd):
		SkbHandler.__init__(self, scbd)
		self.datarow = self.scbd.match.copy()
		
	def getMatches(self):
		prfName = self.scbd.getProfile()
		self.updateScore(self.scbd.getMatches(), self.matches, self.profiles)
		return self.matches[self.profiles[prfName]]
	
class SbdTable(SkbHandler):
	tables = []
	profiles = {}
	
	def __init__(self, scbd):
		SkbHandler.__init__(self, scbd)
		self.datarow = self.scbd.row.copy()
		self.tablerow = self.datarow
		
	def getTable(self):
		prfName = self.scbd.getProfile()
		self.updateScore(self.scbd.getTable(), self.tables, self.profiles)
		return self.tables[self.profiles[prfName]]
		
class Skoreboard(kdeui.KMainWindow, MainDlg):
	title = appname
	icon = "skoreboard"
	tabs = ["Übersicht", "Tabelle"]
	gbs = [["Spiele", "Quicktabelle"],
		   ["Tabelle"]]
	buttons = ["&Aktualisieren", "T&ickerfunktion", "min", "Quelle: "]
	tableCaps = [["Rang", "Mannschaft", "Spiele", "G", "U", "V", "Tore", "Diff.", "Punkte"],
				 ["Heim", "Resultat", "Auswärts"],
				 ["Rang", "Mannschaft", "Diff.", "Punkte"]
				]
	brushes = {"LiveBack" : None, "LiveFore" : None}
	fonts = {"Qualified": None}
	profiles = None
	tblHnd = None
	highlight = []
	
	def __init__(self):		
		kdeui.KDialog.__init__(self)
		self.setupUi(self)
		self.setWindowIcon(kdeui.KIcon(self.icon))
		
		self.brushes["LiveBack"] = QtGui.QBrush(QtGui.QColor(255, 255, 96))
		self.brushes["LiveFore"] = QtGui.QBrush(QtGui.QColor(0, 0, 0))
		self.brushes["ChangedBack"] = QtGui.QBrush(QtGui.QColor(255, 207, 96))
		self.brushes["ChangedFore"] = QtGui.QBrush(QtGui.QColor(0, 0, 0))
		self.fonts["Qualified"] = QtGui.QFont()
		self.fonts["Qualified"].setWeight(QtGui.QFont.Bold)
		self.ticker = QtCore.QTimer(self)
		self.ticker.setInterval(120000)
		self.ticker.start()
		
		self.connect(self.pbRefresh, QtCore.SIGNAL("clicked()"), self.onRefreshBtn)
		self.connect(self.sbTabs, QtCore.SIGNAL("currentChanged(int)"), self.onRefreshBtn)
		self.connect(self.cbProfile, QtCore.SIGNAL("currentIndexChanged(int)"), self.onProfileChng)
		self.connect(self.ticker, QtCore.SIGNAL("timeout()"), self.onTimeout)
		self.connect(self.spInterval, QtCore.SIGNAL("valueChanged(int)"), self.onIntervalChng)
		self.connect(self.cbTicker, QtCore.SIGNAL("stateChanged(int)"), self.onTickerChng)
		self.connect(self.lbSource, QtCore.SIGNAL("linkActivated(QString)"), self.onSourceLb)
		
	def init(self):
		self.scbd = scoreboard.Scoreboard()
		self.tblHnd = SbdTable(self.scbd)
		self.mtcHnd = SbdMatch(self.scbd)
		self.twTable.setColumnCount(len(self.tableCaps[0]))
		self.twMatches.setColumnCount(len(self.tableCaps[1]))
		self.twShortTable.setColumnCount(len(self.tableCaps[2]))
		self.assignCaptions()
		
		self.profiles = self.scbd.getProfiles().items()
		self.profiles.sort()
		for profile in self.profiles:
			self.cbProfile.addItem(profile[1]["name"].decode("utf-8"))
		
		self.scbd.setProfile(self.profiles[0][0])
		self.cbProfile.currentIndex = 0
		
	def resizeEvent(self, event):
		x = 0
		y = 14
		ph = 8	#placeholder
		dw = 13
		dh = 100
		self.sbTabs.setGeometry(self.sbTabs.x(), self.sbTabs.y(), self.centralwidget.width(), self.centralwidget.height() - 60)
		self.gbMatches.setGeometry(x, y, self.centralwidget.width() / 2 - ph, self.centralwidget.height() - dh)
		self.gbShortTable.setGeometry(self.gbMatches.width() + (ph/2), y, self.centralwidget.width() / 2 - ph, self.centralwidget.height() - dh)
		self.gbTable.setGeometry(x, y, self.centralwidget.width() - dw, self.centralwidget.height() - dh)
		
		self.twMatches.setGeometry(10, 30, self.twMatches.parentWidget().width() - 20, self.twMatches.parentWidget().height() - 60)
		self.twShortTable.setGeometry(10, 30, self.twShortTable.parentWidget().width() - 20, self.twShortTable.parentWidget().height() - 60)
		self.twTable.setGeometry(10, 30, self.twTable.parentWidget().width() - 20, self.twTable.parentWidget().height() - 60)
		
		self.lbSource.setGeometry(10, self.centralwidget.height() - 30, self.centralwidget.width() - 20, 40)
			
	def assignCaptions(self):
		self.setWindowTitle(self.title)
		i = 0
		for s in self.tabs:
			self.sbTabs.setTabText(i, s.decode("utf-8"))
			i += 1
		self.gbShortTable.setTitle(self.gbs[0][1].decode("utf-8"))
		self.gbMatches.setTitle(self.gbs[0][0].decode("utf-8"))
		self.gbTable.setTitle(self.gbs[1][0].decode("utf-8"))
		self.pbRefresh.setText(self.buttons[0].decode("utf-8"))
		self.cbTicker.setText(self.buttons[1].decode("utf-8"))
		self.spInterval.setSuffix(self.buttons[2].decode("utf-8"))
		self.lbSource.setText(self.buttons[3].decode("utf-8") + self.scbd.getUrl())
		self.spInterval.setValue(self.ticker.interval() / 60000)
		i = 0
		for cap in self.tableCaps[0]:
			item = QtGui.QTableWidgetItem(cap.decode("utf-8"))
			self.twTable.setHorizontalHeaderItem(i, item)
			i += 1
		i = 0
		for cap in self.tableCaps[1]:
			item = QtGui.QTableWidgetItem(cap.decode("utf-8"))
			self.twMatches.setHorizontalHeaderItem(i, item)
			i += 1
		i = 0
		for cap in self.tableCaps[2]:
			item = QtGui.QTableWidgetItem(cap.decode("utf-8"))
			self.twShortTable.setHorizontalHeaderItem(i, item)
			i += 1
			
	def prepareTableWidget(self, widget, length):
		widget.clearContents()
		widget.setRowCount(length)
		for i in range(length):
			widget.setVerticalHeaderItem(i, QtGui.QTableWidgetItem(""))
			
	def fitSizes(self, widget):
		for i in range(widget.columnCount()):
			widget.resizeColumnToContents(i)
		for i in range(widget.rowCount()):
			widget.resizeRowToContents(i)
		
	def refreshMatches(self):
		matches = self.mtcHnd.getMatches()
		self.prepareTableWidget(self.twMatches, len(matches))
		keys = ["team_home", "result", "team_away"]
		x = 0
		y = 0
		date = False
		for row in matches:
			for x in range(len(self.tableCaps[1])):
				bQualif = False
				if(((row["weekday"] != "") or (row["datetime"] != "")) and not date):
					item = QtGui.QTableWidgetItem(row["weekday"] + ", " + row["datetime"])
					self.twMatches.setItem(y, x, item)
					self.twMatches.setRowCount(self.twMatches.rowCount()+1)
					self.twMatches.setVerticalHeaderItem(self.twMatches.rowCount()-1, QtGui.QTableWidgetItem(""))
					date = True
					y += 1
				if(keys[x] == "team_home" or keys[x] == "team_away"):
					ol = len(row[keys[x]])
					row[keys[x]] = scoreboard.rmTag(row[keys[x]], "b")
					if(ol != len(row[keys[x]])):
						bQualif = True
				item = QtGui.QTableWidgetItem(row[keys[x]])
				if(keys[x] == "result"):
					item.setTextAlignment(QtCore.Qt.AlignCenter)
				if(bQualif):
					item.setFont(self.fonts["Qualified"])
				if(row["status"] == "Live"):
					if(len(row["changed"]) > 0):
						item.setBackground(self.brushes["ChangedBack"])
						item.setForeground(self.brushes["ChangedFore"])
					else:
						item.setBackground(self.brushes["LiveBack"])
						item.setForeground(self.brushes["LiveFore"])
					self.highlight.append(row["team_home"])
					self.highlight.append(row["team_away"])
				self.twMatches.setItem(y, x, item)
			y += 1
			date = False
		self.fitSizes(self.twMatches)
		
	def refreshTable(self, quick = False):
		if(quick):
			qtTable = self.twShortTable
			keys = ["rank", "team", "diff", "points"]
			tci = 2
		else:
			qtTable = self.twTable
			keys = ["rank", "team", "matches", "wins", "draws", "defeats", "scored", "diff", "points"]
			tci = 0
		
		table = self.tblHnd.getTable()
		self.prepareTableWidget(qtTable, len(table))
		
		x = 0
		y = 0
		for row in table:
			for x in range(len(self.tableCaps[tci])):
				if(x != 6):
					item = QtGui.QTableWidgetItem(row[keys[x]])
					if(x != 1):
						item.setTextAlignment(2)
				else:
					s = row[keys[x]] + ":" + row["conceded"]
					item = QtGui.QTableWidgetItem(s)
					item.setTextAlignment(4)
					
				if(keys[x] in row["changed"]):
					item.setBackground(self.brushes["ChangedBack"])
					item.setForeground(self.brushes["ChangedFore"])
				elif(scoreboard.filterTeamName(row["team"]) in self.highlight):
					item.setBackground(self.brushes["LiveBack"])
					item.setForeground(self.brushes["LiveFore"])
				
				qtTable.setItem(y, x, item)
			y += 1
		self.fitSizes(qtTable)
		del self.highlight[0:len(self.highlight)]
	
	def onProfileChng(self, index):
		self.scbd.setProfile(self.profiles[index][0])
		self.onRefreshBtn()
		
	def onRefreshBtn(self):
		self.pbRefresh.setEnabled(False)
		self.setCursor(QtCore.Qt.WaitCursor)
		self.lbSource.setText(self.buttons[3].decode("utf-8") + self.scbd.getUrl())
		if(self.sbTabs.currentIndex() == 0):
			self.refreshMatches()
			self.refreshTable(True)
		elif(self.sbTabs.currentIndex() == 1):
			self.refreshTable()
		self.setCursor(QtCore.Qt.ArrowCursor)
		self.pbRefresh.setEnabled(True)
			
	def onTimeout(self):
		self.onRefreshBtn()
		
	def onIntervalChng(self, interval):
		self.ticker.setInterval(interval * 60000)
		
	def onTickerChng(self, state):
		if(state == 0):
			self.ticker.stop()
		elif(state == 2):
			self.ticker.start()
			
	def onSourceLb(self, link):
		print "klick ", link

kdecore.KCmdLineArgs.init(sys.argv, appname, catalog, kdecore.KLocalizedString(), version)
app = kdeui.KApplication(True)


skbd = Skoreboard()
skbd.init()
skbd.show()
sys.exit(app.exec_())